class Crafting
{
    static _click_events_loaded = false;

    static refresh_table(crafting_table)
    {
        Crafting._refresh_click_events(crafting_table);
        Crafting._refresh_item_display(crafting_table);
        Crafting._refresh_materials_and_level(crafting_table);
        Crafting._refresh_backgrounds(crafting_table);
    }

    static getOilPerBar(ore)
    {
        switch(ore)
        {
            case "copper":
                return 0;
            case "iron":
                return 5;
            case "silver":
                return 15;
            case "gold":
                return 50;
            case "promethium":
                return 100;
            case "titanium":
                return 1000;
            case "ancient_ore":
                return 5000;
            case "dragon_ore":
                return 10000;
        }

        return 0;
    }

    static getCharcoalPerBar(ore)
    {
        switch(ore)
        {
            case "promethium":
                return 5;
            case "titanium":
                return 20;
            case "ancient_ore":
                return 40;
        }

        return 0;
    }

    static getLavaPerBar(ore)
    {
        switch(ore)
        {
            case "titanium":
                return 1;
            case "ancient_ore":
                return 2;
            case "dragon_ore":
                return 5;
        }

        return 0;
    }

    static getPlasmaPerBar(ore)
    {
        switch(ore)
        {
            case "ancient_ore":
                return 1;
            case "dragon_ore":
                return 3;
        }

        return 0;
    }

    static getDragonFirePerBar(ore)
    {
        switch(ore)
        {
            case "dragon_ore":
                return 1;
        }

        return 0;
    }

    static _refresh_click_events(crafting_table)
    {
        if(!Crafting._click_events_loaded)
        {
            var crafting_row_array = crafting_table.getElementsByTagName("tr");
    
            for(var i = 0; i < crafting_row_array.length; i++)
            {
                var crafting_row = crafting_row_array[i];
                if(!crafting_row.hasAttribute("data-crafting-item"))
                    continue;
    
                crafting_row.addEventListener('click', (e) =>
                {
                    var target_clicked = e.target;
                    var tr = target_clicked.closest("tr");
                    var crafting_row_item = tr.getAttribute("data-crafting-item");
                    var can_use_crafting_input_multiple = tr.getAttribute("data-crafting-item-multiple") === "true";

                    if(can_use_crafting_input_multiple)
                        Modals.open_input_dialogue(crafting_row_item, "Crafting", "How many do you want to craft?", "CRAFT");
                    else
                    {
                        var materials = Crafting.get_materials(crafting_row_item);
                        var html = "<div class='modal-crafting-ingredients shadow'>";
                        html += "<b>MATERIALS</b><hr />";
                        for(var i = 0; i < materials.length; i++)
                        {
                            var name = materials[i];
                            i++;
                            var amount = materials[i];

                            var img = '<img width="15px" height="15px" src="https://cdn.idle-pixel.com/images/x.png">';

                            if(Items.getItem(name) >= amount)
                                img = '<img width="15px" height="15px" src="https://cdn.idle-pixel.com/images/check.png">';
                    
                            html += "<img class='w40' src='https://cdn.idle-pixel.com/images/"+name+".png' /> " + "<span class='color-grey'>" + capitalizeFirstLetter(get_item_name(name)) + " x </span>" + format_number(amount) + " " + img;
                            html += "<br />";
                        }
                        html += "</div><br /><br />Craft Item?";

                        document.getElementById("modal-brew-ingredients").innerHTML = html;
                        Modals.open_image_modal("Crafting", "images/" + crafting_row_item + ".png", html, "Craft", "Cancel", "CRAFT=" + crafting_row_item + "~" + 1)
                    }
                        
                });
            }
            Crafting._click_events_loaded = true;
        }
        
    }

    static filterTable(filter) {

        var prefix = "crafting-filter-btn-";
        var btnIds = [
            prefix + "all",
            prefix + "other",
            prefix + "mining",
            prefix + "crafting",
            prefix + "gathering",
            prefix + "farming",
            prefix + "brewing",
            prefix + "fishing",
            prefix + "woodcutting",
            prefix + "cooking",
            prefix + "breeding",
            prefix + "combat",
            prefix + "invention",
        ]
        for(var i = 0; i < btnIds.length; i++) {
            document.getElementById(btnIds[i]).style.backgroundColor = "";
        }
        document.getElementById(prefix + filter).style.backgroundColor = "lime";

        var crafting_table = document.getElementById('crafting-table');
        
        var crafting_row_array = crafting_table.getElementsByTagName("tr");
        for(var i = 0; i < crafting_row_array.length; i++) {
            var row = crafting_row_array[i];

            if(!row.hasAttribute("data-crafting-item"))
                continue;

            if(row.getAttribute('data-crafting-skill') == filter || filter == "all" || row.getAttribute('data-crafting-skill') == "all") {
                row.style.display = '';
            } else {
                row.style.display = 'none';
            }
        }
        Crafting._refresh_item_display(crafting_table)

    }

    static refresh_data_craft_icon_shortcuts() {
        let divs = document.querySelectorAll("[data-craft-shotcut]");
        for(var i = 0; i < divs.length; i++) {
            var div = divs[i];
            var item = div.getAttribute('data-craft-shotcut');
            var level = parseInt(document.querySelector("[data-crafting-item='"+item+"']").getElementsByTagName("item-crafting-table-level")[0].innerHTML);
            if(Items.getItem(item + "_crafted") > 0) {
                div.style.backgroundColor = "green";
            } else if(get_level(Items.getItem('crafting_xp')) >= level) {
                div.style.backgroundColor = "yellow";
            } else {
                div.style.backgroundColor = "pink";
            }
        }
    }

    static _refresh_item_display(crafting_table)
    {
        var crafting_row_array = crafting_table.getElementsByTagName("tr");

        for(var i = 0; i < crafting_row_array.length; i++)
        {
            var crafting_row = crafting_row_array[i];

            if(!crafting_row.hasAttribute("data-crafting-item"))
                continue;
    
            var crafting_row_item = crafting_row.getAttribute("data-crafting-item");

            var max_allowed = 1;
            if(crafting_row.getAttribute("data-miltiple") != null)
                max_allowed = crafting_row.getAttribute("data-miltiple");

            if(Items.getItem(crafting_row_item + "_crafted") >= max_allowed)
            {
                if(max_allowed != -1)
                    crafting_row.style.display = "none";
            }
                
        }
    }

    static _refresh_materials_and_level(crafting_table)
    {
        var materials_req_array = crafting_table.getElementsByTagName("item-crafting-table");
        var levels_req_array = crafting_table.getElementsByTagName("item-crafting-table-level");

        for(var i = 0; i < materials_req_array.length; i++)
        {
            var materials_req = materials_req_array[i];
            var item = materials_req.getAttribute("data-materials-item");

            if(Items.getItem(item) >= parseInt(materials_req.innerHTML))
                materials_req_array[i].style.color = "#00a200";
            else
                materials_req_array[i].style.color = "red";
        }

        for(var i = 0; i < levels_req_array.length; i++)
        {
            var levels_req = levels_req_array[i];
            var level_found = parseInt(levels_req.innerHTML);
            if(get_level(Items.getItem("crafting_xp")) >= level_found)
                levels_req.style.color = "green";
            else
                levels_req.style.color = "red";
        }
    }

    static _refresh_backgrounds(crafting_table)
    {
        var crafting_table_trs = crafting_table.getElementsByTagName("tr");

        for(var i = 0; i < crafting_table_trs.length; i++)
        {
            
            if(!crafting_table_trs[i].hasAttribute("data-crafting-item"))
                continue;

            var materials_req_array = crafting_table_trs[i].getElementsByTagName("item-crafting-table");
            var levels_req_array = crafting_table_trs[i].getElementsByTagName("item-crafting-table-level");

            var can_craft = true;

            for(var j = 0; j < materials_req_array.length; j++)
            {
                var materials_req = materials_req_array[j];
                var item = materials_req.getAttribute("data-materials-item");
    
                if(Items.getItem(item) < parseInt(materials_req.innerHTML))
                    can_craft = false;
            }

            for(var j = 0; j < levels_req_array.length; j++)
            {
                var levels_req = levels_req_array[j];
                var level_found = parseInt(levels_req.innerHTML);
                if(get_level(Items.getItem("crafting_xp")) < level_found)
                    can_craft = false;
            }

            if(can_craft)
            {
                crafting_table_trs[i].style.background = "#e6ffe6";
                crafting_table_trs[i].classList.add("hover")
            }
            else
            {
                crafting_table_trs[i].style.background = "#ffe6e6";
                crafting_table_trs[i].classList.add("hover")
            }
        }
    }

    static getStardustPerXp()
    {
        var stardust_per_xp = Crafting.getHammerToolData()[2];
        if(Items.getItem("blue_hammer_orb_absorbed") > 0)
            stardust_per_xp -= 2;

        if(Achievements.has_completed_set("crafting", "medium"))
            stardust_per_xp -= 1;

        if(
            Items.getItem("crafting_skilling_gloves") > 0 &&
            Items.getItem("crafting_skilling_boots") > 0 &&
            Items.getItem("crafting_skilling_shirt") > 0 &&
            Items.getItem("crafting_skilling_pants") > 0
        )
        stardust_per_xp -= 1;
        
        return stardust_per_xp;
    }

    // returns ["current", "next_gem or none", "stardust per xp"]
    static getHammerToolData()
    {
        if(Items.getItem("diamond_hammer"))
            return ["diamond_hammer", "none", 9];
        
        if(Items.getItem("ruby_hammer"))
            return ["ruby_hammer", "diamond", 10];
        
        if(Items.getItem("emerald_hammer"))
            return ["emerald_hammer", "ruby", 11];
    
        if(Items.getItem("sapphire_hammer"))
            return ["sapphire_hammer", "emerald", 12];

        if(Items.getItem("hammer"))
            return ["hammer", "sapphire", 13];

        return ["none", "none", 13];
    }

    /**
     *  returns an array [item_name, amount, ...]
     */
     static get_materials(item)
     {
         var array = [];
         var entry = $("tr[data-crafting-item='" + item +"']");
         var materials = entry.find("item-crafting-table");
         materials.each(function (ele) {
             var amount = $(this).text();
             var material = $(this).attr("data-materials-item");
             array.push(material, amount);
         });
 
         return array;
     }
}