class TestCanvas
{
  
  static active_hitsplats_monster_map = {};
  static active_canvas_animations_map = {};

  static fps;
  static fpsInterval;
  static startTime;
  static now;
  static then;
  static elapsed;
  static frameCount = 0;

  static paused = false;
  static initialized = false;

  static start(fps) {
    TestCanvas.fps = 0;
    TestCanvas.elapsed = 0;
    TestCanvas.frameCount = 0;
    TestCanvas.fpsInterval = 1000 / fps;
    TestCanvas.then = Date.now();
    TestCanvas.startTime = TestCanvas.then;
    TestCanvas.initialized = true;
    TestCanvas.draw();
  }

  static draw() {
    var ctx = document.getElementById("test-canvas").getContext("2d");
    // stop
    if (Globals.currentPanel != 'panel-canvas-test') {
      return;
    }

    // request another frame
    requestAnimationFrame(TestCanvas.draw);

    // calc elapsed time since last loop
    TestCanvas.now = Date.now();
    TestCanvas.elapsed = TestCanvas.now - TestCanvas.then;

    // if enough time has elapsed, draw the next frame
    if (TestCanvas.elapsed > TestCanvas.fpsInterval) {

      // Get ready for next frame by setting then=now, but...
      // Also, adjust for fpsInterval not being multiple of 16.67
      TestCanvas.then = TestCanvas.now - (TestCanvas.elapsed % TestCanvas.fpsInterval);

      // draw stuff here
      ctx.clearRect(0, 0, 800, 600);
      ctx.beginPath();
      ctx.rect(rand(0, 600), rand(0, 600), 150, 100);
      ctx.fill();

      // fps counter
      var sinceStart = TestCanvas.now - TestCanvas.startTime;
      var currentFps = Math.round(1000 / (sinceStart / ++TestCanvas.frameCount) * 100) / 100;

      ctx.fillText("FPS:" + currentFps, 20, 20);
    }
  }

  static addAnimation(path, start_x, start_y, end_x, end_y, timeMS)
  {

      var obj = new TestAnimationObject(path, start_x, start_y, end_x, end_y, timeMS);
      var random_key = rand(1,500000);
      TestCanvas.active_canvas_animations_map[random_key] = obj;

      setTimeout(
          function(){
              delete TestCanvas.active_canvas_animations_map[random_key];
          },
          timeMS
      )
  }

  static addHitSplatMonster(label, icon, label_color, background_color, border_color, team_member_username)
  {
      var splat = new TestHitSplat(
        label,
        icon,
        label_color,
        background_color,
        border_color,
        400,
        300,
      );
      var random_key = rand(1,500000);
      TestCanvas.active_hitsplats_monster_map[random_key] = splat;

      setTimeout(
          function(){
              delete TestCanvas.active_hitsplats_monster_map[random_key];
          },
          1000
      )
  }

  
}




class TestAnimationObject {

  // Raids.addCanvasAnimationObject("images/projectile_1.png", 545, 358, 40, 500, 1000)
  // Raids.addCanvasAnimationObject("images/projectile_1.png", 545, 358, 40, 350, 1000)
  // Raids.addCanvasAnimationObject("images/projectile_1.png", 545, 358, 40, 200, 1000)
  // Raids.addCanvasAnimationObject("images/projectile_1.png", 545, 358, 40, 50, 1000)


  constructor(path, start_x, start_y, end_x, end_y, timeMS) {
    this.path = path;
    this.start_x = parseInt(start_x);
    this.start_y = parseInt(start_y);
    this.end_x = parseInt(end_x);
    this.end_y = parseInt(end_y);
    this.timeMS = parseInt(timeMS);

    this.at_x = this.start_x;
    this.at_y = this.start_y;

    var diff_x = Math.abs(this.start_x - this.end_x);
    var diff_y = Math.abs(this.start_y - this.end_y);

    this.increment_x = diff_x / 60 / (timeMS/1000);
    this.increment_y = diff_y / 60 / (timeMS/1000);

    this.x_orientation = -1;
    this.y_orientation = 1;
    if(this.start_y > this.end_y) {
      this.y_orientation = -1;
    }
  }

  draw(ctx)
  {
    this.at_x += this.increment_x * this.x_orientation;
    this.at_y += this.increment_y * this.y_orientation;

    ctx.drawImage(
      Cache.getImage(
        this.path,
        this.path,
      ), 
      this.at_x,
      this.at_y,
    );
  }
}

class TestHitSplat
{
    constructor(label, icon, label_color, background_color, border_color, start_x, start_y) 
    {
        this.label = label;
        this.icon = icon;
        this.label_color = label_color;
        this.background_color = background_color;
        this.border_color = border_color;

        this.start_x = parseFloat(start_x);
        this.start_y = parseFloat(start_y);
        this.offset_x = 0 - Math.random() + Math.random();
        this.offset_y = 0 - Math.random() + Math.random();
        //this.offset_x = 0; // no diversions for now.
        //this.offset_y = 0; // no diversions for now.
    }

    draw(ctx)
    {
        ctx.font = "20px pixel";
        var hit_splat_width = 60;
        var hit_splat_height = 30;
        hit_splat_width = ctx.measureText(this.label).width
        hit_splat_width += 40

        
        ctx.strokeStyle = "black";
        ctx.beginPath();
        ctx.rect(this.start_x, this.start_y, hit_splat_width, hit_splat_height);
        ctx.stroke();
        ctx.fillStyle = this.background_color;
        ctx.fillRect(this.start_x,this.start_y,hit_splat_width,hit_splat_height);
        ctx.drawImage(Cache.getImage(this.icon, "raids_hitsplat_" + this.icon), this.start_x, this.start_y, 30, 30);
        ctx.font = "20px pixel";
        ctx.fillStyle = this.label_color;
        ctx.fillText(this.label, this.start_x + 35, this.start_y + 22);
        this.start_x += this.offset_x;
        this.start_y += this.offset_y;
        this.start_y-=0.5;
    }
}